#!/usr/bin/python
# -*- coding: utf-8 -*-

# thumbor imaging service
# https://github.com/thumbor/thumbor/wiki

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2011 globo.com thumbor@googlegroups.com

BUILTIN_EXTENSIONS = [
    "thumbor.ext.filters._alpha",
    "thumbor.ext.filters._bounding_box",
    "thumbor.ext.filters._brightness",
    "thumbor.ext.filters._colorize",
    "thumbor.ext.filters._composite",
    "thumbor.ext.filters._contrast",
    "thumbor.ext.filters._convolution",
    "thumbor.ext.filters._curve",
    "thumbor.ext.filters._equalize",
    "thumbor.ext.filters._fill",
    "thumbor.ext.filters._nine_patch",
    "thumbor.ext.filters._noise",
    "thumbor.ext.filters._rgb",
    "thumbor.ext.filters._round_corner",
    "thumbor.ext.filters._saturation",
    "thumbor.ext.filters._sharpen",
]
